/*
 * <header and disclaimer>
 *
 *
 *
 */

#include "gpio_lib.h"

/* 
 * Function:
 *   GPIO_Register()
 * Parameters:
 *   iGpio - GPIO pin number
 * Description:
 *   Registers pin iGpio as GPIO in /sys/class/gpio
 * Returns:
 *   1 on success, -1 on error
 */
int GPIO_Register (int iGpio)
{
	/* Open export file which does the actual registering */
	iFd = open("/sys/class/gpio/export", O_WRONLY|O_SYNC);
	if (0 > iFd)
	{
		/* In case of an error: print message and leave function */
		printf("Cannot open export file.\n");
		return -1;
	}

	/* When successfully opened continue: */
	/* Convert the iGpio integer to string to allow interaction in sysfs */
	snprintf(caValue, sizeof(caValue), "%d", iGpio);

	/* Write the string to the opened export file */
	if (1 > write(iFd,&caValue,strlen(caValue)))
	{
		printf("Cannot write to the export file.\n");
		/* Close the file before leaving */
		close(iFd);
		return -1;
	}

	/* When no error occured we can close the export file */
	close(iFd);

	/* Now check to see if the GPIO folder was created */
	snprintf(caPath, sizeof(caPath), "/sys/class/gpio/gpio%d/value", iGpio);

	iFd = open(caPath, O_RDONLY);
	if (0 > iFd)
	{
		printf("Could not open GPIO directory to check existence.\n");
		return -1;
	}

	close(iFd);

	/* When the file exists, the GPIO registration was successful */
	return 1;
}

/* 
 * Function:
 *   GPIO_Free()
 * Parameters:
 *   iGpio - GPIO pin number
 * Description:
 *   Unregisters pin iGpio from /sys/class/gpio
 * Returns:
 *   1 on success, -1 on error
 */
int GPIO_Free(int iGpio)
{
	/* Open the unexport file which does the actual unregistering */
	iFd = open("/sys/class/gpio/unexport", O_WRONLY|O_SYNC);
	if (0 > iFd)
	{
		printf("Cannot open unexport file.\n");
		return -1;
	}

	snprintf(caValue, sizeof(caValue), "%d", iGpio);

	if (1 > write(iFd,&caValue,strlen(caValue)))
	{
		printf("Cannot write to the unexport file.\n");
		close(iFd);
		return -1;
	}

	close(iFd);

	return 1;
}

/* 
 * Function:
 *   GPIO_SetVal()
 * Parameters:
 *   iGpio - GPIO pin number
 *   iValue - Value to assign to the pin (0 or 1)
 * Description:
 *   Assigns a value to the pin
 * Returns:
 *   1 on success, -1 on error
 */
int GPIO_SetVal(int iGpio, int iValue)
{
	snprintf(caPath, sizeof(caPath), "/sys/class/gpio/gpio%d/value", iGpio);

	iFd = open(caPath, O_WRONLY|O_SYNC);
	if (0 > iFd)
	{
		printf("Cannot open value file.\n");
		return -1;
	}

	snprintf(caValue, sizeof(caValue), "%d", iValue);

	if (1 > write(iFd, &caValue, strlen(caValue)))
	{
		printf("Cannot write to value file.\n");
		close(iFd);
		return -1;
	}

	close(iFd);

	return 1;
}

/* 
 * Function:
 *   GPIO_SetDir()
 * Parameters:
 *   iGpio - GPIO pin number
 *   iDirection - Data direction of the pin (1 = output, 0 = input)
 * Description:
 *   Sets a data direction to a pin
 * Returns:
 *   1 on success, -1 on error
 */
int GPIO_SetDir(int iGpio, int iDirection)
{
	snprintf(caPath, sizeof(caPath), "/sys/class/gpio/gpio%d/direction", iGpio);

	iFd = open(caPath, O_WRONLY|O_SYNC);
	if (0 > iFd)
	{
		printf("Cannot open direction file.\n");
		return -1;
	}

	/* When iDirection is 1, then the pin is set to output */
	if (1 == iDirection)
		snprintf(caValue, sizeof(caValue), "out");
	else
		snprintf(caValue, sizeof(caValue), "in");

	if (1 > write(iFd, &caValue, strlen(caValue)))
	{
		printf("Cannot write to direction file.\n");
		close(iFd);
		return -1;
	}

	close(iFd);

	return 1;
}

/* 
 * Function:
 *   GPIO_GetVal()
 * Parameters:
 *   iGpio - GPIO pin number
 * Description:
 *   Returns the pin value of a GPIO
 * Returns:
 *   Pin value on success, -1 on error
 */
int GPIO_GetVal(int iGpio)
{
	snprintf(caPath, sizeof(caPath), "/sys/class/gpio/gpio%d/value", iGpio);

	iFd = open(caPath, O_RDONLY|O_SYNC);

	if (0 > iFd)
	{
		printf("Cannot open value file.\n");
		return -1;
	}

	/* Read 1 byte from the value file, either an ASCII 0 or 1 */
	read(iFd, &caValue, 1);

	close(iFd);

	/* Convert the ASCII character to integer and return it */
	return atoi(caValue);
}

/* 
 * Function:
 *   GPIO_Init()
 * Parameters:
 *   iGpio - GPIO pin number
 *   iValue - Value to assign to the pin
 * Description:
 *   Registers a GPIO as input or output depending on iValue.
 *   iValues of 0 or 1 set the pin to output and assign it to 0 or 1.
 *   Other values of iValue set the pin to input and return current value.
 * Returns:
 *   iValue = 0 or 1: 1 on success, -1 on failure
 *   iValue = others: Pin value on success, -1 on failure
 */
int GPIO_Init(int iGpio, int iValue)
{
	int ret = -1;

	if (GPIO_Register (iGpio))
	{
		if ((0 == iValue) || (1 == iValue))
		{
			GPIO_SetDir(iGpio, 1);
			ret = GPIO_SetVal(iGpio, iValue);
		}
		else
			GPIO_SetDir(iGpio, 0);
			ret = GPIO_GetVal(iGpio);
	}

	return ret;
}

/* 
 * Function:
 *   GPIO_Toggle()
 * Parameters:
 *   iGpio - GPIO pin number
 * Description:
 *   Toggles the current value of the GPIO pin
 * Returns:
 *   nothing
 */
void GPIO_Toggle(int iGpio)
{
	GPIO_SetVal(iGpio, !GPIO_GetVal(iGpio));
}

